@@ -20,6 +20,7 @@ class Settings(BaseSettings):
     APP_NAME: str = "AuthTuna"
     ALGORITHM: str = "HS256"  # JWT Encryption algorithm
     API_BASE_URL: str
+    TRY_FULL_INITIALIZE_WHEN_SYSTEM_USER_EXISTS_AGAIN: bool = False
     # Security settings
     JWT_SECRET_KEY: SecretStr = SecretStr("dev-secret-key-change-in-production")
     ENCRYPTION_PRIMARY_KEY: SecretStr = SecretStr("dev-encryption-key-change-in-production")
@@ -31,8 +32,8 @@ class Settings(BaseSettings):
     DEFAULT_SUPERADMIN_EMAIL: str = "superadmin@example.com"
     DEFAULT_ADMIN_EMAIL: str = "admin@example.com"
     # Database settings
-    DEFAULT_DATABASE_URI: str = "sqlite+aiosqlite:///./authtuna_dev.db"  # JUST PROVIDE SYNC URI and set async to true if you want to use async db
-    DATABASE_USE_ASYNC_ENGINE: bool = True  # WILL replace sqlite with sqlite+aiosqlite and asyncpg with asyncpg+asyncpg (if true)
+    DEFAULT_DATABASE_URI: str = "sqlite+aiosqlite:///./authtuna_dev.db"  # PROVIDE ASYNC URI
+    DATABASE_USE_ASYNC_ENGINE: bool = True  # dosent do anything
     AUTO_CREATE_DATABASE: bool = True  # Automatically create the database tables if they don't exist
     DATABASE_POOL_SIZE: int = 20
     DATABASE_MAX_OVERFLOW: int = 40
@@ -50,7 +51,9 @@ class Settings(BaseSettings):
     SESSION_SECURE: bool = True  # obvio its gon be httponly coz it auth bruh so not letting anyone config dat.
     SESSION_TOKEN_NAME: str = "session_token"
     SESSION_COOKIE_DOMAIN: Optional[str] = None
-
+    LOCK_SESSION_REGION: bool = True  # Lock session to region based on IP geolocation, depending on security requirements and environments you may want to disable this.
+    DISABLE_RANDOM_STRING: bool = False  # IN ENVIRONMENTS WHERE YOU HAVE LONG RUNNING CONNECTIONS, AND HIGH CONCURRENCY, DISABLING THIS WILL HELP PREVENT LOGOUTS DUE TO RANDOM STRING MISMATCH.
+    RANDOM_STRING_GRACE: int = 300  # seconds, STORED RANDOM STRINGS IN THIS ROLLING TIMEFRAME WILL BE ACCEPTED.
     # Email settings (disabled by default)
     EMAIL_ENABLED: bool = False
     SMTP_HOST: Optional[str] = None
@@ -72,6 +75,8 @@ class Settings(BaseSettings):
 
     EMAIL_TEMPLATE_DIR: str = os.path.join(module_path, "templates/email")
     HTML_TEMPLATE_DIR: str = os.path.join(module_path, "templates/pages")
+    DASHBOARD_AND_USER_INFO_PAGES_TEMPLATE_DIR: str = os.path.join(module_path, "templates/dashboard")
+
     # OAuth settings
     GOOGLE_CLIENT_ID: Optional[str] = None
     GOOGLE_CLIENT_SECRET: Optional[SecretStr] = None
@@ -81,6 +86,28 @@ class Settings(BaseSettings):
     GITHUB_CLIENT_SECRET: Optional[SecretStr] = None
     GITHUB_REDIRECT_URI: Optional[str] = None
 
+    # RPC settings
+    RPC_ENABLED: bool = False  # was working on it, didnt work so left for now will get back to it later, parts of this removed
+    RPC_AUTOSTART: bool = True  # If enabled then will autostart by default ()
+    RPC_TOKEN: SecretStr = SecretStr("changeme-secure-token")
+    RPC_TLS_CERT_FILE: Optional[str] = None
+    RPC_TLS_KEY_FILE: Optional[str] = None
+    RPC_ADDRESS: str = "[::]:50051"
+
+    # Webauthn settings
+    WEBAUTHN_ENABLED: bool = False  # Currently NOT IMPLEMENTED.
+    # WEBAUTHN_AUTHENTICATOR_ATTACHMENT: Optional[str]  # "platform" or "cross-platform"
+    # WEBAUTHN_USER_VERIFICATION: Optional[str]  # "required" or "preferred" or "discouraged"
+    # WEBAUTHN_PUBLIC_KEY_CREDENTIALS_CHARSET: Optional[str] = "ascii"
+    # WEBAUTHN_PUBLIC_KEY_CREDENTIALS_LENGTH: int = 32
+    # WEBAUTHN_PUBLIC_KEY_CREDENTIALS_COUNT: int = 10
+    WEBAUTHN_RP_ID: str = "localhost"  # The domain of your site
+    WEBAUTHN_RP_NAME: str = "AuthTuna"
+    WEBAUTHN_ORIGIN: str = "http://localhost:8000"
+
+    # Authentication strategy: "COOKIE" or "BEARER"
+    STRATEGY: str = "COOKIE"  # Options: "COOKIE", "BEARER"
+
     model_config = SettingsConfigDict(env_file=os.getenv("ENV_FILE_NAME", ".env"), env_file_encoding='utf-8',
                                       extra='ignore')
 
@@ -92,12 +119,13 @@ def init_settings(**kwargs: Any) -> "Settings":
     testing or when using a secrets manager.
 
     Args:
-        **kwargs: Keyword arguments to override settings from the environment.
+        **kwargs: Keyword arguments to initialize settings with.
+        If you want to override specific settings in your environment, pass USE_ENV=True with params to override.
     """
-    global _settings_instance
+    global _settings_instance, use_env
     if _settings_instance is not None:
         logger.warning("Settings have already been initialized. Re-initializing.")
-    # The standard constructor loads from env/.env first, then overrides with kwargs.
+    use_env = kwargs.get("use_env", False)
     _settings_instance = Settings(**kwargs)
     return _settings_instance
 
@@ -113,17 +141,14 @@ def get_settings() -> "Settings":
 
     global _settings_instance
     if _settings_instance is None:
-        # Check if the user has explicitly disabled auto-initialization
         if use_env:
             raise RuntimeError(
                 "AUTHTUNA_NO_ENV is set. Settings must be initialized manually "
                 "by calling `init_settings()` at application startup."
             )
         else:
-            # Auto-initialize for backward compatibility and simple use cases.
             logger.debug("Auto-initializing settings on first access.")
-            _settings_instance = init_settings()
-
+            _settings_instance = init_settings(use_env=True)
     return _settings_instance
 
 
@@ -134,9 +159,8 @@ def get_settings() -> "Settings":
 # This provides the just-in-time, conditional initialization.
 
 class _SettingsProxy:
-    def __getattr__(self, name: str) -> Any:
+    def __getattr__(self, name: str) -> Any:  # WAS TRYING TO ADD TYPEHINT SO AUTOCOMPLETE WILL WORK FOR `settings.VAR` BUT ITS NOT WORKING IF SOMEONE KNOWS HLP.
         return getattr(get_settings(), name)
 
 
 settings = _SettingsProxy()
-
