@@ -10,7 +10,7 @@ import time
 from contextlib import asynccontextmanager
 from typing import AsyncGenerator, Optional
 import asyncio
-from sqlalchemy import Column, event, Table, ForeignKey, text, AsyncAdaptedQueuePool, VARCHAR
+from sqlalchemy import Column, event, Table, ForeignKey, text, AsyncAdaptedQueuePool, VARCHAR, LargeBinary
 from sqlalchemy.dialects.postgresql import CITEXT, JSONB
 from sqlalchemy.engine import Engine
 from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession, async_sessionmaker
@@ -146,6 +146,21 @@ role_grant_permissions = Table(
 )
 
 
+organization_members = Table(
+    'organization_members', Base.metadata,
+    Column('user_id', String(64), ForeignKey('users.id'), primary_key=True),
+    Column('organization_id', String(64), ForeignKey('organization.id'), primary_key=True),
+    Column('joined_at', Float, default=time.time, nullable=False)
+)
+
+
+team_members = Table(
+    'team_members', Base.metadata,
+    Column('user_id', String(64), ForeignKey('users.id'), primary_key=True),
+    Column('team_id', String(64), ForeignKey('team.id'), primary_key=True),
+    Column('joined_at', Float, default=time.time, nullable=False)
+)
+
 # --- ORM Models ---
 
 class User(Base):
@@ -175,12 +190,16 @@ class User(Base):
         secondaryjoin=lambda: Role.id == user_roles_association.c.role_id,
     )
 
+    passkey_credentials = relationship("PasskeyCredential", back_populates="user", cascade="all, delete-orphan")
     sessions = relationship("Session", back_populates="user", cascade="all, delete-orphan")
     tokens = relationship("Token", back_populates="user", cascade="all, delete-orphan", foreign_keys="Token.user_id")
     social_accounts = relationship("SocialAccount", back_populates="user", cascade="all, delete-orphan")
     mfa_methods = relationship("MFAMethod", back_populates="user", cascade="all, delete-orphan")
     mfa_recovery_codes = relationship("MFARecoveryCode", back_populates="user", cascade="all, delete-orphan")
     audit_events = relationship("AuditEvent", back_populates="user", cascade="all, delete-orphan")
+    organizations_owned = relationship("Organization", back_populates="owner", foreign_keys="Organization.owner_id")
+    organizations = relationship("Organization", secondary=organization_members, back_populates="members")
+    teams = relationship("Team", secondary=team_members, back_populates="members")
 
     async def set_password(self, password: str, ip: str, db_manager_custom=None, db: AsyncSession = None):
         """
@@ -300,19 +319,19 @@ class Session(Base):
     create_ip = Column(String(45))
     last_ip = Column(String(45))
     user = relationship('User', back_populates='sessions')
-    random_string = Column(String(255), nullable=False, default=encryption_utils.gen_random_string, onupdate=encryption_utils.gen_random_string)
+    random_string = Column(String(255), nullable=False, default=encryption_utils.gen_random_string)
     previous_random_strings = Column(JsonType, nullable=False, default=list)
 
     def is_expired(self):
         return time.time() > self.etime or time.time() > self.e_abs_time
 
-    async def is_valid(self, region: str = "", device: str = "", random_string: str = "", db: AsyncSession = None) -> bool:
+    async def is_valid(self, region: str = "", device: str = "", random_string: str = "", ip: str = "", db: AsyncSession = None) -> bool:
         """
         Checks if the session is valid, not expired, and matches device/region/random_string.
         Invalidates and logs if any check fails.
         """
         if self.active and not self.is_expired():
-            if self.region != region:
+            if settings.LOCK_SESSION_REGION and (self.region != region)  and (self.create_ip != ip):
                 await db_manager.log_audit_event(self.user_id, "SESSION_INVALIDATED", self.last_ip, {"reason": "region_mismatch"}, db=db)
                 await self.terminate(self.last_ip, db=db)
                 return False
@@ -320,7 +339,10 @@ class Session(Base):
                 await db_manager.log_audit_event(self.user_id, "SESSION_INVALIDATED", self.last_ip, {"reason": "device_mismatch"}, db=db)
                 await self.terminate(self.last_ip, db=db)
                 return False
-            is_token_valid = (self.random_string == random_string or
+            if settings.DISABLE_RANDOM_STRING:
+                is_token_valid = True
+            else:
+                is_token_valid = (self.random_string == random_string or
                               any(entry['value'] == random_string for entry in self.previous_random_strings))
 
             if is_token_valid:
@@ -362,9 +384,11 @@ class Session(Base):
         now = time.time()
 
         new_history = [{'value': self.random_string, 'timestamp': now}]
-
-        grace_period = settings.SESSION_DB_VERIFICATION_INTERVAL + 5
-        for entry in self.previous_random_strings:
+        if settings.DISABLE_RANDOM_STRING:
+            grace_period = 0
+        else:
+            grace_period = settings.SESSION_DB_VERIFICATION_INTERVAL + settings.RANDOM_STRING_GRACE # HARDCODED AS REQUESTS MAY RUN FOR LIKE 60 SEC USUALLY BEFORE BEING PROCESSED BY SERVER AND TIMEOUT AFTER THAT, MINIMIZES FALSE INVALIDATIONS
+        for entry in self.previous_random_strings:                    # INCREASED GRACE PERIOD TO 92 SECS FROM 60 SECS TO HANDLE SSE SCENARIOS BETTER.
             if now - entry.get('timestamp', 0) < grace_period:
                 new_history.append(entry)
         self.previous_random_strings = new_history
@@ -439,6 +463,24 @@ class MFARecoveryCode(Base):
     user = relationship('User', back_populates='mfa_recovery_codes')
 
 
+class PasskeyCredential(Base):
+    __tablename__ = 'passkey_credentials'
+
+    id = Column(LargeBinary(1023), primary_key=True)
+    user_id = Column(String(64), ForeignKey('users.id', ondelete="CASCADE"), nullable=False, index=True)
+    nickname = Column(String(100), nullable=False)
+    public_key = Column(LargeBinary, nullable=False)
+    sign_count = Column(Integer, default=0, nullable=False)
+    aaguid = Column(LargeBinary(16), nullable=True)
+    is_discoverable = Column(Boolean, default=False, nullable=False)
+    is_backup_eligible = Column(Boolean, default=False, nullable=False)
+    is_backed_up = Column(Boolean, default=False, nullable=False)
+    transports = Column(JsonType, nullable=True, default=list)
+    created_at = Column(Float, nullable=False, default=time.time)
+    last_used_at = Column(Float, nullable=False, default=time.time, onupdate=time.time)
+    user = relationship("User", back_populates="passkey_credentials")
+
+
 class DeletedUser(Base):
     __tablename__ = 'deleted_users'
     user_id = Column(String(64), primary_key=True)
@@ -459,6 +501,37 @@ class AuditEvent(Base):
     user = relationship('User', back_populates='audit_events')
 
 
+class Organization(Base):
+    __tablename__ = 'organization'
+
+    id = Column(String(64), primary_key=True, default=lambda: f"org_{encryption_utils.gen_random_string(28)}")
+    name = Column(String(100), nullable=False)
+    owner_id = Column(String(64), ForeignKey('users.id'), nullable=False)
+    created_at = Column(Float, nullable=False, default=time.time)
+
+    owner = relationship("User", back_populates="organizations_owned", foreign_keys=[owner_id])
+    teams = relationship("Team", back_populates="organization", cascade="all, delete-orphan")
+    members = relationship("User", secondary=organization_members, back_populates="organizations")
+
+    def __repr__(self):
+        return f"<Organization {self.name} (ID: {self.id})>"
+
+
+class Team(Base):
+    __tablename__ = 'team'
+
+    id = Column(String(64), primary_key=True, default=lambda: f"team_{encryption_utils.gen_random_string(27)}")
+    name = Column(String(100), nullable=False)
+    organization_id = Column(String(64), ForeignKey('organization.id'), nullable=False)
+    created_at = Column(Float, nullable=False, default=time.time)
+
+    organization = relationship("Organization", back_populates="teams")
+    members = relationship("User", secondary=team_members, back_populates="teams")
+
+    def __repr__(self):
+        return f"<Team {self.name} (ID: {self.id})>"
+
+
 class DatabaseManager:
     """
     Manages async database connections and sessions for AuthTuna.
